#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include "palace.h"

#warning FIXME: get rid of send_ok global variable
extern gboolean send_ok;


void
on_palace_confdialog_destroy           (GtkObject       *object,
                                        gpointer         user_data)
{
	debug_printf("Configuration dialog received delete event\n");

	palace_confdialog = NULL;
}


void
on_send_lpt_data_check_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	extern GtkWidget *palace_renderwindow;
	if((palacecfg.send_lpt_data = gtk_toggle_button_get_active(togglebutton))) {
		if(!send_ok)
			send_ok = (palace_control_init() ? FALSE : TRUE);
		if(!send_ok) {
			fprintf(stderr, "Palace was unable to initialize parallel port output\n");
			gtk_window_set_title(GTK_WINDOW(palace_renderwindow), "Palace -- Port Inactive");
		} else {
			gtk_window_set_title(GTK_WINDOW(palace_renderwindow), "Palace -- Port Active");
		}
	} else {
		send_ok = FALSE;
		palace_control_cleanup();
		gtk_window_set_title(GTK_WINDOW(palace_renderwindow), "Palace -- Port Inactive");
	}
}


void
on_lpt_address_changed                 (GtkEditable     *editable,
                                        gpointer         user_data)
{
	GtkToggleButton *togglebutton;
	gchar address[7];
	int lpt_address = 0;
	
	strcpy(address, gtk_entry_get_text(GTK_ENTRY(editable)));

	togglebutton = GTK_TOGGLE_BUTTON(lookup_widget(palace_confdialog, "send_lpt_data_check"));

	sscanf(address, "%i", &lpt_address);
	
	if(lpt_address != palacecfg.lpt_address) {
		debug_printf("Palace: port address changed, closing parallel port\n");

		if(togglebutton) {
			gtk_toggle_button_set_active(togglebutton, FALSE);
		} else {
			if(send_ok) {
				send_ok = FALSE;
				palace_control_cleanup();
				palacecfg.send_lpt_data = FALSE;
			}
		}

		palacecfg.lpt_address = lpt_address;
	}
}


void
on_draw_graphics_check_toggled         (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if((palacecfg.draw_graphics = gtk_toggle_button_get_active(togglebutton)))
		palace_playback_start_internal();
	else
		palace_playback_stop_internal();
}


void
on_reverse_channel_order_check_toggled (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	debug_printf("Reverse channel order checkbox toggled\n");
	palacecfg.reverse_channel_order = gtk_toggle_button_get_active(togglebutton);
}


void
on_flash_on_beats_check_toggled        (GtkToggleButton *togglebutton,
					gpointer         user_data)
{
	debug_printf("Flash on beats checkbox toggled\n");
	palacecfg.flash_on_beats = gtk_toggle_button_get_active(togglebutton);
}


void
on_vu_radio_toggled                    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active(togglebutton)) {
		debug_printf("Switched to VU mode\n");
		palacecfg.mode = VU;
	}
}


void
on_spectrum_radio_toggled              (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active(togglebutton)) {
		debug_printf("Switched to SPECTRUM mode\n");
		palacecfg.mode = SPECTRUM;
	}
}


void
on_stepper_radio_toggled               (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	if(gtk_toggle_button_get_active(togglebutton)) {
		debug_printf("Switched to STEPPER mode\n");
		palacecfg.mode = STEPPER;
	}
}


gboolean
on_pixmap1_button_release_event        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	debug_printf("Button released over the logo pixmap\n");

	return FALSE;
}


void
on_all_channels_hscale_value_changed   (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	gint i, channel_value, delta;
	gint all_value, old_value;
	gchar buf[20];
	GtkAdjustment *adjustment = NULL;

	all_value = GTK_ADJUSTMENT(widget)->value;
	debug_printf("Value of all channels slider changed to %d\n", all_value);
	old_value = (int)gtk_object_get_data(GTK_OBJECT(widget), "old_value");
	debug_printf("old_value of object at 0x%08lx is set to %i.\n", (unsigned long)GTK_OBJECT(widget), old_value);
	delta = all_value - old_value;
	debug_printf("delta is %i.\n", delta);
	gtk_object_set_data(GTK_OBJECT(widget), "old_value", (gpointer)all_value);
	
	if(delta) {
		for(i = 0; i < NUM_BANDS; i++) {
			channel_value = palacecfg.threshold[i] + delta;
			
			if(channel_value < 1)
				channel_value = 1;
			if(channel_value > MAX_SENSE)
				channel_value = MAX_SENSE;
			
			sprintf(buf, "channel_%i_hscale", i + 1);
			adjustment = GTK_RANGE(lookup_widget(palace_confdialog, buf))->adjustment;
			gtk_adjustment_set_value(adjustment, channel_value);
		}
	}
}


void
on_channel_1_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 1 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);

	palacecfg.threshold[0] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_2_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 2 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);

	palacecfg.threshold[1] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_3_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 3 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);
	
	palacecfg.threshold[2] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_4_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 4 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);
	
	palacecfg.threshold[3] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_5_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 5 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);
	
	palacecfg.threshold[4] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_6_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 6 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);
	
	palacecfg.threshold[5] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_7_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 7 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);
	
	palacecfg.threshold[6] = GTK_ADJUSTMENT(widget)->value;
}


void
on_channel_8_hscale_value_changed      (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of channel 8 slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);
	
	palacecfg.threshold[7] = GTK_ADJUSTMENT(widget)->value;
}


void
on_frequency_falloff_hscale_value_changed
                                        (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of falloff slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);

	palacecfg.frequency_falloff_rate = GTK_ADJUSTMENT(widget)->value;
}


void
on_variable_frequency_bars_check_toggled
                                        (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	debug_printf("Variable frequency bars checkbox toggled\n");

	palacecfg.variable_frequency_bars = gtk_toggle_button_get_active(togglebutton);
}


void
on_stepper_speed_hscale_value_changed  (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	debug_printf("Value of stepper_speed slider changed to %f\n", GTK_ADJUSTMENT(widget)->value);

	palacecfg.stepper_strobe_rate = 10 - (int)GTK_ADJUSTMENT(widget)->value;
}


void
on_stepper_vary_speed_check_toggled    (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{
	debug_printf("Stepper motor variable speed check toggled\n");

	palacecfg.stepper_vary_speed = gtk_toggle_button_get_active(togglebutton);

	if(!palacecfg.stepper_vary_speed) {
		/* If switching "vary speed" off, restore the speed to its proper value */
		palacecfg.stepper_strobe_rate = 10 - (int)GTK_RANGE(lookup_widget(palace_confdialog, "stepper_speed_hscale"))->adjustment->value;
	}
}


void
on_ok_button_clicked                   (GtkButton       *button,
                                        gpointer         user_data)
{
	debug_printf("OK button clicked\n");
	palace_write_config();
	gtk_widget_destroy(palace_confdialog);
}


void
on_revert_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	debug_printf("Revert button clicked\n");
	palace_read_config();
}


void
on_cancel_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	debug_printf("Cancel button clicked\n");
	palace_read_config();
	gtk_widget_destroy(palace_confdialog);
}


void
on_vu_power_hscale_value_changed       (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	palacecfg.vu_power_scale = GTK_ADJUSTMENT(widget)->value;

	debug_printf("VU power hscale value changed to %f\n", palacecfg.vu_power_scale);
}


void
on_target_bpm_hscale_value_changed     (GtkWidget       *widget,
                                        GtkStateType     state,
                                        gpointer         user_data)
{
	palacecfg.target_bpm = (int)GTK_ADJUSTMENT(widget)->value;

	debug_printf("Target BPM hscale value changed to %u\n", palacecfg.target_bpm);
}

