/* palace-config.c - global configuration for Palace DCI
 *
 * $Id: palace-config.c, v1.0 2002/06/07 18:25:16 nitroglycerine Exp $
 * 
 *     This file contains functions related to configuration files and
 * options.
 */

#include "palace.h"

/* Global configuration variables */
palace_config_t palacecfg;

#if !HAVE_XMMS_CFG_GET_DEFAULT_FILENAME
static gchar *xmms_cfg_get_default_filename(void)
{
	static gchar *filename = NULL;

	ENTER_FUNCTION();

	if (!filename)
	{
		filename = g_strconcat(g_get_home_dir(), "/.xmms/config", NULL);
	}

	LEAVE_FUNCTION();
	
	return filename;
}
#endif

#if !HAVE_XMMS_CFG_OPEN_DEFAULT_FILE
static ConfigFile *xmms_cfg_open_default_file(void)
{
	ConfigFile *ret;

	ENTER_FUNCTION();

	ret = xmms_cfg_open_file(xmms_cfg_get_default_filename());
	if (!ret)
		ret = xmms_cfg_new();

	LEAVE_FUNCTION();
	
	return ret;
}
#endif

#if !HAVE_XMMS_CFG_WRITE_DEFAULT_FILE
static gboolean xmms_cfg_write_default_file(ConfigFile *cfg)
{
	ENTER_FUNCTION();
	LEAVE_FUNCTION();
	return xmms_cfg_write_file(cfg, xmms_cfg_get_default_filename());
}
#endif

/* Read configuration options from the xmms configuration file */

void palace_read_config()
{
	gint i;
	ConfigFile *cf;

	ENTER_FUNCTION();
	
	/* Initialize values to defaults */
	/* general */
	palacecfg.draw_graphics = TRUE;
	palacecfg.reverse_channel_order = FALSE;
	palacecfg.mode = VU;

	/* frequency spectrum */
	palacecfg.variable_frequency_bars = TRUE;
	palacecfg.frequency_falloff_rate = 6;
	for (i = 0; i < NUM_BANDS; i++)
		palacecfg.threshold[i] = (DEFAULTSENSE - 5) * (NUM_BANDS - i - 1) / NUM_BANDS + 5;

	/* vu motor */
	palacecfg.vu_power_scale = 0.125;
	
	/* stepper motor */
	palacecfg.stepper_pins = 8;
	palacecfg.stepper_strobe_rate = 6;

	/* parallel port */
	palacecfg.send_lpt_data = FALSE;
	palacecfg.lpt_address = DEFAULTLPTPORT;

	/* beat detection */
	palacecfg.flash_on_beats = TRUE;
	palacecfg.target_bpm = 142;

	/* Open configuration file */
	if ((cf = xmms_cfg_open_default_file())) {
		/* Global configuration options */
		xmms_cfg_read_boolean(cf, XMMS_CFG_SECTION, "draw_graphics", &palacecfg.draw_graphics);
		xmms_cfg_read_boolean(cf, XMMS_CFG_SECTION, "reverse_channel_order", &palacecfg.reverse_channel_order);
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "mode", (int *)&palacecfg.mode);

		/* Frequency spectrum options */
		xmms_cfg_read_boolean(cf, XMMS_CFG_SECTION, "variable_frequency_bars", &palacecfg.variable_frequency_bars);
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "frequency_falloff_rate", &palacecfg.frequency_falloff_rate);
		for (i = 0; i < NUM_BANDS; i++) {
			gchar thresholdtext[20];
			sprintf(thresholdtext, "threshold%d", i);
			xmms_cfg_read_int(cf, XMMS_CFG_SECTION, thresholdtext,
				&palacecfg.threshold[i]);
		}

		/* Vu-meter options */
		xmms_cfg_read_float(cf, XMMS_CFG_SECTION, "vu_power_scale", &palacecfg.vu_power_scale);
		
		/* Stepper motor options */
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "stepper_pins", &palacecfg.stepper_pins);
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "stepper_strobe_rate", &palacecfg.stepper_strobe_rate);
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "stepper_vary_speed", &palacecfg.stepper_vary_speed);

		/* Parallel port output options */
		xmms_cfg_read_boolean(cf, XMMS_CFG_SECTION, "send_lpt_data", &palacecfg.send_lpt_data);
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "lpt_address", &palacecfg.lpt_address);

		/* Beat detection options */
		xmms_cfg_read_boolean(cf, XMMS_CFG_SECTION, "flash_on_beats", &palacecfg.flash_on_beats);
		xmms_cfg_read_int(cf, XMMS_CFG_SECTION, "target_bpm", &palacecfg.target_bpm);
		palace_state.mean_time_between_flashes = 60.0 / palacecfg.target_bpm;


		/* Close the configuration file */
		xmms_cfg_free(cf);
	}
	
	/* Set the values of the widgets in the configuration dialog (if it is open) to those just read */
	init_config_values();

	LEAVE_FUNCTION();
}

void palace_write_config()
{
	int i;
	ConfigFile *cf;

	ENTER_FUNCTION();
	
	/* Open the configuration file */
	if((cf = xmms_cfg_open_default_file())) {
		/* Global configuration options */
		xmms_cfg_write_boolean(cf, XMMS_CFG_SECTION, "draw_graphics", palacecfg.draw_graphics);
		xmms_cfg_write_boolean(cf, XMMS_CFG_SECTION, "reverse_channel_order", palacecfg.reverse_channel_order);
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "mode", palacecfg.mode);

		/* Frequency spectrum options */
		xmms_cfg_write_boolean(cf, XMMS_CFG_SECTION, "variable_frequency_bars", palacecfg.variable_frequency_bars);
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "frequency_falloff_rate", palacecfg.frequency_falloff_rate);
		for (i = 0; i < NUM_BANDS; i++) {
			gchar thresholdtext[20];
			sprintf(thresholdtext, "threshold%d", i);
			xmms_cfg_write_int(cf, XMMS_CFG_SECTION, thresholdtext, palacecfg.threshold[i]);
		}

		/* Vu-meter options */
		xmms_cfg_write_float(cf, XMMS_CFG_SECTION, "vu_power_scale", palacecfg.vu_power_scale);
		
		/* Stepper motor options */
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "stepper_pins", palacecfg.stepper_pins);
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "stepper_strobe_rate", palacecfg.stepper_strobe_rate);
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "stepper_vary_speed", palacecfg.stepper_vary_speed);

		/* Parallel port output options */
		xmms_cfg_write_boolean(cf, XMMS_CFG_SECTION, "send_lpt_data", palacecfg.send_lpt_data);
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "lpt_address", palacecfg.lpt_address);

		/* Beat detection options */
		xmms_cfg_write_boolean(cf, XMMS_CFG_SECTION, "flash_on_beats", palacecfg.flash_on_beats);
		xmms_cfg_write_int(cf, XMMS_CFG_SECTION, "target_bpm", palacecfg.target_bpm);

		/* Flush values to disk */
		xmms_cfg_write_default_file(cf);

		/* Close configuration file */
		xmms_cfg_free(cf);
	} else {
		/* Something is wrong - cf == NULL */
		debug_printf("Unable to open xmms configuration file for writing\n");
	}

	LEAVE_FUNCTION();
}


